@@ -14,6 +14,9 @@
 
 #include "hfst-ol.h"
 #include <string>
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
 
 namespace hfst_ol {
 
@@ -33,61 +36,144 @@ void skip_c_string(char ** raw)
     ++(*raw);
 }
 
+bool is_big_endian()
+{
+#ifdef WORDS_BIGENDIAN
+    return true;
+#else
+    return false;
+#endif
+}
+
+uint16_t read_uint16_flipping_endianness(FILE * f)
+{
+    uint16_t result = 0;
+    unsigned char byte1 = static_cast<uint8_t>(getc(f));
+    unsigned char byte2 = static_cast<uint8_t>(getc(f));
+    result |= byte2;
+    result <<= 8;
+    result |= byte1;
+    return result;
+}
+
+uint16_t read_uint16_flipping_endianness(char * raw)
+{
+    uint16_t result = 0;
+    result |= *(raw + 1);
+    result <<= 8;
+    result |= *raw;
+    return result;
+}
+
+uint32_t read_uint32_flipping_endianness(FILE * f)
+{
+    uint32_t result = 0;
+    unsigned char byte1 = static_cast<uint8_t>(getc(f));
+    unsigned char byte2 = static_cast<uint8_t>(getc(f));
+    unsigned char byte3 = static_cast<uint8_t>(getc(f));
+    unsigned char byte4 = static_cast<uint8_t>(getc(f));
+    result |= byte4;
+    result <<= 8;
+    result |= byte3;
+    result <<= 8;
+    result |= byte2;
+    result <<= 8;
+    result |= byte1;
+    return result;
+}
+
+uint32_t read_uint32_flipping_endianness(char * raw)
+{
+    uint32_t result = 0;
+    result |= *(raw + 3);
+    result <<= 8;
+    result |= *(raw + 2);
+    result <<= 8;
+    result |= *(raw + 1);
+    result <<= 8;
+    result |= *raw;
+    return result;
+}
+
+float read_float_flipping_endianness(FILE * f)
+{
+    union {
+        float f;
+        char c[4];
+    } flipper;
+
+    (void)f;
+    flipper.f = 0;
+
+    return flipper.f;
+}
+
 void
 TransducerHeader::read_property(bool& property, FILE* f)
 {
-    unsigned int prop;
-    if (fread(&prop,sizeof(unsigned int),1,f) != 1) {
-        HFST_THROW_MESSAGE(HeaderParsingException,
-                           "Header ended unexpectedly\n");
-    }
-    if (prop == 0)
-    {
-        property = false;
-        return;
-    }
-    else
-    {
-        property = true;
-        return;
+    if (is_big_endian()) {
+        // four little-endian bytes
+        int c = getc(f);
+        property = !(c == 0);
+        c = getc(f);
+        c = getc(f);
+        c = getc(f);
+    } else {
+        unsigned int prop;
+        if (fread(&prop,sizeof(uint32_t),1,f) != 1) {
+            HFST_THROW_MESSAGE(HeaderParsingException,
+                               "Header ended unexpectedly\n");
+        }
+        if (prop == 0)
+        {
+            property = false;
+        }
+        else
+        {
+            property = true;
+        }
     }
 }
 
 void
 TransducerHeader::read_property(bool& property, char** raw)
 {
-    unsigned int prop = *((unsigned int *) *raw);
-    (*raw) += sizeof(unsigned int);
-    if (prop == 0)
-    {
-        property = false;
-        return;
-    }
-    else
-    {
-        property = true;
-        return;
+    if (is_big_endian()) {
+        property = !(**raw == 0);
+    } else {
+        unsigned int prop = *((unsigned int *) *raw);
+        property = !(prop == 0);
     }
+    (*raw) += sizeof(uint32_t);
 }
 
 void TransducerHeader::skip_hfst3_header(FILE * f)
 {
     const char* header1 = "HFST";
     unsigned int header_loc = 0; // how much of the header has been found
-    int c;
+    int c = 0;
     for(header_loc = 0; header_loc < strlen(header1) + 1; header_loc++)
     {
         c = getc(f);
         if(c != header1[header_loc]) {
+            //std::cerr << header_loc << ": " << int(c) << " != " << header1[header_loc] << std::endl;
             break;
         }
     }
     if(header_loc == strlen(header1) + 1) // we found it
     {
-        unsigned short remaining_header_len;
-        if (fread(&remaining_header_len,
-                  sizeof(remaining_header_len), 1, f) != 1 ||
-            getc(f) != '\0') {
+        uint16_t remaining_header_len = 0;
+        if (is_big_endian()) {
+            remaining_header_len = read_uint16_flipping_endianness(f);
+        } else {
+            if (fread(&remaining_header_len,
+                      sizeof(remaining_header_len), 1, f) != 1) {
+                HFST_THROW_MESSAGE(HeaderParsingException,
+                                   "Found broken HFST3 header\n");
+            }
+        }
+        //std::cerr << "remaining_header_len " << remaining_header_len << std::endl;
+        if (getc(f) != '\0') {
             HFST_THROW_MESSAGE(HeaderParsingException,
                                "Found broken HFST3 header\n");
         }
@@ -106,7 +192,7 @@ void TransducerHeader::skip_hfst3_header(FILE * f)
         if (type_field != std::string::npos) {
             if (header_tail.find("HFST_OL") != type_field + 5 &&
                 header_tail.find("HFST_OLW") != type_field + 5) {
-                delete headervalue;
+                delete[] headervalue;
                 HFST_THROW_MESSAGE(
                     TransducerTypeException,
                     "Transducer has incorrect type, should be "
@@ -131,14 +217,21 @@ void TransducerHeader::skip_hfst3_header(char ** raw)
     for(header_loc = 0; header_loc < strlen(header1) + 1; header_loc++)
     {
         if(**raw != header1[header_loc]) {
+            //std::cerr << header_loc << ": " << int(**raw) << " != " << header1[header_loc] << std::endl;
             break;
         }
         ++(*raw);
     }
     if(header_loc == strlen(header1) + 1) // we found it
     {
-        unsigned short remaining_header_len = *((unsigned short *) *raw);
-        (*raw) += sizeof(unsigned short) + 1 + remaining_header_len;
+        uint16_t remaining_header_len = 0;
+        if (is_big_endian()) {
+            remaining_header_len = read_uint16_flipping_endianness(*raw);
+        } else {
+            remaining_header_len = *((unsigned short *) *raw);
+        }
+        //std::cerr << "remaining_header_len " << remaining_header_len << std::endl;
+        (*raw) += sizeof(uint16_t) + 1 + remaining_header_len;
     } else // nope. put back what we've taken
     {
         --(*raw); // first the non-matching character
@@ -152,23 +245,41 @@ void TransducerHeader::skip_hfst3_header(char ** raw)
 TransducerHeader::TransducerHeader(FILE* f)
 {
     skip_hfst3_header(f); // skip header iff it is present
-    /* The following conditional clause does all the numerical reads
-       and throws an exception if any fails to return 1 */
-    if (fread(&number_of_input_symbols,
-              sizeof(SymbolNumber),1,f) != 1||
-        fread(&number_of_symbols,
-              sizeof(SymbolNumber),1,f) != 1||
-        fread(&size_of_transition_index_table,
-              sizeof(TransitionTableIndex),1,f) != 1||
-        fread(&size_of_transition_target_table,
-              sizeof(TransitionTableIndex),1,f) != 1||
-        fread(&number_of_states,
-              sizeof(TransitionTableIndex),1,f) != 1||
-        fread(&number_of_transitions,
-              sizeof(TransitionTableIndex),1,f) != 1) {
-        HFST_THROW_MESSAGE(HeaderParsingException,
-                           "Header ended unexpectedly\n");
+    if (is_big_endian()) {
+        // no error checking yet
+        number_of_input_symbols = read_uint16_flipping_endianness(f);
+        number_of_symbols = read_uint16_flipping_endianness(f);
+        size_of_transition_index_table = read_uint32_flipping_endianness(f);
+        size_of_transition_target_table = read_uint32_flipping_endianness(f);
+        number_of_states = read_uint32_flipping_endianness(f);
+        number_of_transitions = read_uint32_flipping_endianness(f);
+    } else {
+        /* The following conditional clause does all the numerical reads
+           and throws an exception if any fails to return 1 */
+        if (fread(&number_of_input_symbols,
+                  sizeof(SymbolNumber),1,f) != 1||
+            fread(&number_of_symbols,
+                  sizeof(SymbolNumber),1,f) != 1||
+            fread(&size_of_transition_index_table,
+                  sizeof(TransitionTableIndex),1,f) != 1||
+            fread(&size_of_transition_target_table,
+                  sizeof(TransitionTableIndex),1,f) != 1||
+            fread(&number_of_states,
+                  sizeof(TransitionTableIndex),1,f) != 1||
+            fread(&number_of_transitions,
+                  sizeof(TransitionTableIndex),1,f) != 1) {
+            HFST_THROW_MESSAGE(HeaderParsingException,
+                               "Header ended unexpectedly\n");
+        }
     }
+    /*
+    std::cerr << "number_of_input_symbols " << number_of_input_symbols << std::endl;
+    std::cerr << "number_of_symbols " << number_of_symbols << std::endl;
+    std::cerr << "size_of_transition_index_table " << size_of_transition_index_table << std::endl;
+    std::cerr << "size_of_transition_target_table " << size_of_transition_target_table << std::endl;
+    std::cerr << "number_of_states " << number_of_states << std::endl;
+    std::cerr << "number_of_transitions " << number_of_transitions << std::endl;
+    //*/
     read_property(weighted,f);
     read_property(deterministic,f);
     read_property(input_deterministic,f);
@@ -183,18 +294,41 @@ TransducerHeader::TransducerHeader(FILE* f)
 TransducerHeader::TransducerHeader(char** raw)
 {
     skip_hfst3_header(raw); // skip header iff it is present
-    number_of_input_symbols = *(SymbolNumber*) *raw;
-    (*raw) += sizeof(SymbolNumber);
-    number_of_symbols = *(SymbolNumber*) *raw;
-    (*raw) += sizeof(SymbolNumber);
-    size_of_transition_index_table = *(TransitionTableIndex*) *raw;
-    (*raw) += sizeof(TransitionTableIndex);
-    size_of_transition_target_table = *(TransitionTableIndex*) *raw;
-    (*raw) += sizeof(TransitionTableIndex);
-    number_of_states = *(TransitionTableIndex*) *raw;
-    (*raw) += sizeof(TransitionTableIndex);
-    number_of_transitions = *(TransitionTableIndex*) *raw;
-    (*raw) += sizeof(TransitionTableIndex);
+    if (is_big_endian()) {
+        number_of_input_symbols = read_uint16_flipping_endianness(*raw);
+        (*raw) += sizeof(SymbolNumber);
+        number_of_symbols = read_uint16_flipping_endianness(*raw);
+        (*raw) += sizeof(SymbolNumber);
+        size_of_transition_index_table = read_uint32_flipping_endianness(*raw);
+        (*raw) += sizeof(TransitionTableIndex);
+        size_of_transition_target_table = read_uint32_flipping_endianness(*raw);
+        (*raw) += sizeof(TransitionTableIndex);
+        number_of_states = read_uint32_flipping_endianness(*raw);
+        (*raw) += sizeof(TransitionTableIndex);
+        number_of_transitions = read_uint32_flipping_endianness(*raw);
+        (*raw) += sizeof(TransitionTableIndex);
+    } else {
+        number_of_input_symbols = *(SymbolNumber*) *raw;
+        (*raw) += sizeof(SymbolNumber);
+        number_of_symbols = *(SymbolNumber*) *raw;
+        (*raw) += sizeof(SymbolNumber);
+        size_of_transition_index_table = *(TransitionTableIndex*) *raw;
+        (*raw) += sizeof(TransitionTableIndex);
+        size_of_transition_target_table = *(TransitionTableIndex*) *raw;
+        (*raw) += sizeof(TransitionTableIndex);
+        number_of_states = *(TransitionTableIndex*) *raw;
+        (*raw) += sizeof(TransitionTableIndex);
+        number_of_transitions = *(TransitionTableIndex*) *raw;
+        (*raw) += sizeof(TransitionTableIndex);
+    }
+    /*
+    std::cerr << "number_of_input_symbols " << number_of_input_symbols << std::endl;
+    std::cerr << "number_of_symbols " << number_of_symbols << std::endl;
+    std::cerr << "size_of_transition_index_table " << size_of_transition_index_table << std::endl;
+    std::cerr << "size_of_transition_target_table " << size_of_transition_target_table << std::endl;
+    std::cerr << "number_of_states " << number_of_states << std::endl;
+    std::cerr << "number_of_transitions " << number_of_transitions << std::endl;
+    //*/
     read_property(weighted,raw);
     read_property(deterministic,raw);
     read_property(input_deterministic,raw);
@@ -209,24 +343,24 @@ TransducerHeader::TransducerHeader(char** raw)
 SymbolNumber
 TransducerHeader::symbol_count()
 {
-    return number_of_symbols; 
+    return number_of_symbols;
 }
 
 SymbolNumber
 TransducerHeader::input_symbol_count()
 {
-    return number_of_input_symbols; 
+    return number_of_input_symbols;
 }
 TransitionTableIndex
-TransducerHeader::index_table_size(void)
+TransducerHeader::index_table_size()
 {
-    return size_of_transition_index_table; 
+    return size_of_transition_index_table;
 }
 
-TransitionTableIndex 
+TransitionTableIndex
 TransducerHeader::target_table_size()
 {
-    return size_of_transition_target_table; 
+    return size_of_transition_target_table;
 }
 
 bool
@@ -258,7 +392,7 @@ TransducerHeader::probe_flag(HeaderFlag flag)
 bool
 FlagDiacriticOperation::isFlag() const
 {
-    return feature != NO_SYMBOL; 
+    return feature != NO_SYMBOL;
 }
 
 FlagDiacriticOperator
@@ -305,7 +439,7 @@ void TransducerAlphabet::read(FILE * f, SymbolNumber number_of_symbols)
             if (byte == EOF) {
                 HFST_THROW(AlphabetParsingException);
             }
-            *sym = byte;
+            *sym = static_cast<char>(byte);
             ++sym;
         }
         *sym = 0;
@@ -366,7 +500,7 @@ void TransducerAlphabet::read(FILE * f, SymbolNumber number_of_symbols)
         string_to_symbol[std::string(line)] = k;
     }
     free(line);
-    flag_state_size = feature_bucket.size();
+    flag_state_size = static_cast<SymbolNumber>(feature_bucket.size());
 }
 
 void TransducerAlphabet::read(char ** raw, SymbolNumber number_of_symbols)
@@ -443,7 +577,7 @@ void TransducerAlphabet::read(char ** raw, SymbolNumber number_of_symbols)
         string_to_symbol[std::string(*raw)] = k;
         skip_c_string(raw);
     }
-    flag_state_size = feature_bucket.size();
+    flag_state_size = static_cast<SymbolNumber>(feature_bucket.size());
 }
 
 TransducerAlphabet::TransducerAlphabet(FILE* f, SymbolNumber number_of_symbols):
@@ -465,7 +599,7 @@ TransducerAlphabet::TransducerAlphabet(char** raw,
 
 void TransducerAlphabet::add_symbol(std::string & sym)
 {
-    string_to_symbol[sym] = kt.size();
+    string_to_symbol[sym] = static_cast<SymbolNumber>(kt.size());
     kt.push_back(sym);
 }
 
@@ -478,7 +612,7 @@ void TransducerAlphabet::add_symbol(char * sym)
 KeyTable*
 TransducerAlphabet::get_key_table()
 {
-    return &kt; 
+    return &kt;
 }
 
 OperationMap*
@@ -535,6 +669,9 @@ void IndexTable::read(FILE * f,
     if (fread(indices,table_size, 1, f) != 1) {
         HFST_THROW(IndexTableReadingException);
     }
+    if (is_big_endian()) {
+        convert_to_big_endian();
+    }
 }
 
 void IndexTable::read(char ** raw,
@@ -544,6 +681,23 @@ void IndexTable::read(char ** raw,
     indices = (char*)(malloc(table_size));
     memcpy((void *) indices, (const void *) *raw, table_size);
     (*raw) += table_size;
+    if (is_big_endian()) {
+        convert_to_big_endian();
+    }
+}
+
+void IndexTable::convert_to_big_endian()
+{
+    //std::cerr << "IndexTable::convert_to_big_endian " << size << std::endl;
+    // We have a sequence of uint_16's and uint_32's in memory,
+    // and we're going to flip endianness of each one.
+    for(size_t i = 0; i < size; ++i) {
+        // first the uint_16
+        std::swap(*(indices + i * TransitionIndex::SIZE + 0), *(indices + i * TransitionIndex::SIZE + 1));
+        // then the uint_32
+        std::swap(*(indices + i * TransitionIndex::SIZE + 2), *(indices + i * TransitionIndex::SIZE + 5));
+        std::swap(*(indices + i * TransitionIndex::SIZE + 3), *(indices + i * TransitionIndex::SIZE + 4));
+    }
 }
 
 void TransitionTable::read(FILE * f,
@@ -554,6 +708,9 @@ void TransitionTable::read(FILE * f,
     if (fread(transitions, table_size, 1, f) != 1) {
         HFST_THROW(TransitionTableReadingException);
     }
+    if (is_big_endian()) {
+        convert_to_big_endian();
+    }
 }
 
 void TransitionTable::read(char ** raw,
@@ -563,6 +720,27 @@ void TransitionTable::read(char ** raw,
     transitions = (char*)(malloc(table_size));
     memcpy((void *) transitions, (const void *) *raw, table_size);
     (*raw) += table_size;
+    if (is_big_endian()) {
+        convert_to_big_endian();
+    }
+}
+
+void TransitionTable::convert_to_big_endian()
+{
+    //std::cerr << "TransitionTable::convert_to_big_endian " << size << std::endl;
+    // We have a sequence of [ uint_16, uint_16, uint_32, float ] in memory,
+    // and we're going to flip endianness of each one.
+    for(size_t i = 0; i < size; ++i) {
+        // first the uint_16s
+        std::swap(*(transitions + i * Transition::SIZE + 0), *(transitions + i * Transition::SIZE + 1));
+        std::swap(*(transitions + i * Transition::SIZE + 2), *(transitions + i * Transition::SIZE + 3));
+        // then the uint_32
+        std::swap(*(transitions + i * Transition::SIZE + 4), *(transitions + i * Transition::SIZE + 7));
+        std::swap(*(transitions + i * Transition::SIZE + 5), *(transitions + i * Transition::SIZE + 6));
+        // then the float
+        std::swap(*(transitions + i * Transition::SIZE + 8), *(transitions + i * Transition::SIZE + 11));
+        std::swap(*(transitions + i * Transition::SIZE + 9), *(transitions + i * Transition::SIZE + 10));
+    }
 }
 
 void LetterTrie::add_string(const char * p, SymbolNumber symbol_key)
@@ -602,14 +780,14 @@ LetterTrie::~LetterTrie()
          i != letters.end(); ++i)
     {
         if (*i)
-        { 
+        {
             delete *i;
         }
     }
 }
 
 Encoder::Encoder(KeyTable * kt, SymbolNumber number_of_input_symbols):
-    ascii_symbols(UCHAR_MAX,NO_SYMBOL)
+    ascii_symbols(UCHAR_MAX+1,NO_SYMBOL)
 {
     read_input_symbols(kt, number_of_input_symbols);
 }
@@ -621,9 +799,9 @@ void Encoder::read_input_symbol(const char * s, const int s_num)
     }
     if ((strlen(s) == 1) && (unsigned char)(*s) <= 127)
     {
-        ascii_symbols[(unsigned char)(*s)] = s_num;
+        ascii_symbols[(unsigned char)(*s)] = static_cast<SymbolNumber>(s_num);
     }
-    letters.add_string(s, s_num);
+    letters.add_string(s, static_cast<SymbolNumber>(s_num));
 }
 
 void Encoder::read_input_symbol(std::string const & s, const int s_num)
@@ -647,15 +825,15 @@ TransitionIndex::target() const
     return first_transition_index;
 }
 
-bool 
-TransitionIndex::final(void) const
+bool
+TransitionIndex::final() const
 {
     return input_symbol == NO_SYMBOL &&
         first_transition_index != NO_TABLE_INDEX;
 }
 
 Weight
-TransitionIndex::final_weight(void) const
+TransitionIndex::final_weight() const
 {
     union to_weight
     {
@@ -667,37 +845,37 @@ TransitionIndex::final_weight(void) const
 }
 
 SymbolNumber
-TransitionIndex::get_input(void) const
+TransitionIndex::get_input() const
 {
     return input_symbol;
 }
 
-TransitionTableIndex 
-Transition::target(void) const
+TransitionTableIndex
+Transition::target() const
 {
     return target_index;
 }
 
 SymbolNumber
-Transition::get_output(void) const
+Transition::get_output() const
 {
     return output_symbol;
 }
 
 SymbolNumber
-Transition::get_input(void) const
+Transition::get_input() const
 {
     return input_symbol;
 }
 
 Weight
-Transition::get_weight(void) const
+Transition::get_weight() const
 {
     return transition_weight;
 }
 
 bool
-Transition::final(void) const
+Transition::final() const
 {
     return input_symbol == NO_SYMBOL &&
         output_symbol == NO_SYMBOL &&
@@ -711,7 +889,7 @@ IndexTable::IndexTable(FILE* f,
 {
     read(f, number_of_table_entries);
 }
-    
+
 IndexTable::IndexTable(char ** raw,
                        TransitionTableIndex number_of_table_entries):
     indices(NULL),
@@ -720,7 +898,7 @@ IndexTable::IndexTable(char ** raw,
     read(raw, number_of_table_entries);
 }
 
-IndexTable::~IndexTable(void)
+IndexTable::~IndexTable()
 {
     if (indices) {
         free(indices);
@@ -742,7 +920,7 @@ TransitionTableIndex
 IndexTable::target(TransitionTableIndex i) const
 {
     if (i < size) {
-        return hfst_deref((TransitionTableIndex *) 
+        return hfst_deref((TransitionTableIndex *)
                           (indices + TransitionIndex::SIZE * i +
                            sizeof(SymbolNumber)));
     } else {
@@ -775,7 +953,7 @@ TransitionTable::TransitionTable(FILE * f,
 {
     read(f, transition_count);
 }
-  
+
 TransitionTable::TransitionTable(char ** raw,
                                  TransitionTableIndex transition_count):
     transitions(NULL),
@@ -784,7 +962,7 @@ TransitionTable::TransitionTable(char ** raw,
     read(raw, transition_count);
 }
 
-TransitionTable::~TransitionTable(void)
+TransitionTable::~TransitionTable()
 {
     if (transitions) {
         free(transitions);
