@@ -37,27 +37,6 @@ int nByte_utf8(unsigned char c)
     }
 }
 
-bool
-StringWeightComparison::operator()(StringWeightPair lhs, StringWeightPair rhs)
-{ // return true when we want rhs to appear before lhs
-    if (reverse) {
-        return (lhs.second < rhs.second);
-    } else {
-        return (lhs.second > rhs.second);
-    }
-}
-
-bool
-StringPairWeightComparison::operator()(StringPairWeightPair lhs,
-                                       StringPairWeightPair rhs)
-{ // return true when we want rhs to appear before lhs
-    if (reverse) {
-        return (lhs.second < rhs.second);
-    } else {
-        return (lhs.second > rhs.second);
-    }
-}
-
 void WeightQueue::push(Weight w)
 {
     for (WeightQueue::iterator it = begin(); it != end(); ++it) {
@@ -238,7 +217,7 @@ Speller::Speller(Transducer* mutator_ptr, Transducer* lexicon_ptr):
 SymbolNumber
 Speller::get_state_size()
 {
-    return lexicon->get_state_size();
+    return static_cast<SymbolNumber>(lexicon->get_state_size());
 }
 
 
@@ -280,7 +259,14 @@ void Speller::lexicon_consume(void)
         // no more input
         return;
     }
-    SymbolNumber this_input = alphabet_translator[input[input_state]];
+    SymbolNumber this_input;
+    if (mutator != NULL) {
+        this_input = alphabet_translator[input[input_state]];
+    } else {
+        // To support zhfst spellers without error models, we allow
+        // for the case with plain lexicon symbols
+        this_input = input[input_state];
+    }
     if(!lexicon->has_transitions(
            next_node.lexicon_state + 1, this_input)) {
         // we have no regular transitions for this
@@ -681,7 +667,7 @@ bool Transducer::has_non_epsilons_or_flags(const TransitionTableIndex i)
         return((this_input != 0 && this_input != NO_SYMBOL) &&
                !is_flag(this_input));
     } else {
-        SymbolNumber max_symbol = get_key_table()->size();
+        SymbolNumber max_symbol = static_cast<SymbolNumber>(get_key_table()->size());
         for (SymbolNumber sym = 1; sym < max_symbol; ++sym) {
             if (indices.input_symbol(i + sym) == sym) {
                 return true;
@@ -756,6 +742,7 @@ Transducer::is_weighted(void)
 
 AnalysisQueue Speller::analyse(char * line, int nbest)
 {
+    (void)nbest;
     mode = Lookup;
     if (!init_input(line)) {
         return AnalysisQueue();
@@ -1090,7 +1077,7 @@ void Speller::build_alphabet_translator(void)
             // A symbol in the error source isn't present in the
             // lexicon, so we add it.
             std::string sym = from_keys->operator[](i);
-            SymbolNumber lexicon_key = lexicon->get_key_table()->size();
+            SymbolNumber lexicon_key = static_cast<SymbolNumber>(lexicon->get_key_table()->size());
             lexicon->get_encoder()->read_input_symbol(sym, lexicon_key);
             lexicon->get_alphabet()->add_symbol(sym);
             alphabet_translator.push_back(lexicon_key);
@@ -1115,10 +1102,14 @@ bool Speller::init_input(char * line)
     SymbolNumber k = NO_SYMBOL;
     char ** inpointer = &line;
     char * oldpointer;
-    
+
     while (**inpointer != '\0') {
         oldpointer = *inpointer;
-        k = mutator->get_encoder()->find_key(inpointer);
+        if (mutator != NULL) {
+            k = mutator->get_encoder()->find_key(inpointer);
+        } else {
+            k = lexicon->get_encoder()->find_key(inpointer);
+        }
         if (k == NO_SYMBOL) { // no tokenization from alphabet
             int bytes_to_tokenize = nByte_utf8(static_cast<unsigned char>(*oldpointer));
             if (bytes_to_tokenize == 0) {
@@ -1137,14 +1128,16 @@ bool Speller::init_input(char * line)
                 SymbolNumber k_lexicon = lexicon->get_alphabet()->get_string_to_symbol()
                     ->operator[](new_symbol_string);
                 lexicon->get_encoder()->read_input_symbol(new_symbol, k_lexicon);
-                if (!mutator->get_alphabet()->has_string(new_symbol_string)) {
-                    mutator->get_alphabet()->add_symbol(new_symbol_string);
-                }
-                k = mutator->get_alphabet()->get_string_to_symbol()->
-                    operator[](new_symbol_string);
-                mutator->get_encoder()->read_input_symbol(new_symbol, k);
-                if (k >= alphabet_translator.size()) {
-                    add_symbol_to_alphabet_translator(k_lexicon);
+                if (mutator != NULL) {
+                    if (!mutator->get_alphabet()->has_string(new_symbol_string)) {
+                        mutator->get_alphabet()->add_symbol(new_symbol_string);
+                    }
+                    k = mutator->get_alphabet()->get_string_to_symbol()->
+                        operator[](new_symbol_string);
+                    mutator->get_encoder()->read_input_symbol(new_symbol, k);
+                    if (k >= alphabet_translator.size()) {
+                        add_symbol_to_alphabet_translator(k_lexicon);
+                    }
                 }
                 input.push_back(k);
                 continue;
